/*
 * Created on 24 juin 2004
 * NewSale.java
 * Représente une vente.
 */
package org.progx.salesmanager.ui;

import java.awt.CardLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.table.AbstractTableModel;

import org.progx.salesmanager.event.QuitListener;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;


/**
 * Représente une vente, graphiquement et logiquement.
 * 
 * @author Romain Guy
 */
public class NewSale implements QuitListener {

    private SalesManagerFrame parent;
    private AnimatedPanel connectionAnimation;
    private JPanel cardPane;
    private CardLayout carder;
    private JLabel info;
    private JButton saleButton;
    private SalesModel model;
    private JTextField soldTo;
    private JTextField channel;
    private JTextField division;
    private JTextField organization;
    private JTextField orderType;
    private JTextField purchase;
    private InfiniteProgressPanel glassPane;

    public NewSale(SalesManagerFrame parent) {
        this.parent = parent;
        this.parent.addQuitListener(this);
        this.glassPane = new InfiniteProgressPanel();
        this.parent.setGlassPane(glassPane);
    }

    /**
     * Crée un nouvel ordre de vente.
     * 
     * @author Romain Guy
     */
    private void perform() {
        try {
            Thread.sleep(4000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        displaySuccess("You've successfully waited :)");

        glassPane.stop();
        saleButton.setEnabled(true);
    }

    /**
     * Affiche un message de succès.
     * 
     * @param text Le message à afficher
     */
    private void displaySuccess(String text) {
        info.setIcon(UIHelper.readImageIcon("success.png"));
        info.setText(text);
    }

    /**
     * Affiche un message d'avertissement.
     * 
     * @param text Le message à afficher
     */
    private void displayWarning(String text) {
        info.setIcon(UIHelper.readImageIcon("warning.png"));
        info.setText(text);
    }

    /**
     * Affiche un message d'erreur.
     * 
     * @param text Le message à afficher
     */
    private void displayFailure(String text) {
        info.setIcon(UIHelper.readImageIcon("failure.png"));
        info.setText(text);
    }

    /**
     * Crée un formulaire de saisie de commande dans un JPanel.
     * 
     * @return Un panneau contenant le formulaire de saisie d'une commande
     * @author Romain Guy
     */
    public Component buildPanel() {
        FormLayout layout =
            new FormLayout("right:pref, 3dlu, pref, 7dlu, right:pref, 3dlu, pref", // colonnes
                "p, 3dlu, p, 3dlu, p, 3dlu, p, 3dlu, p, 9dlu, fill:default," + // lignes
                "3dlu, pref, 3dlu, pref"); // lignes
        layout.setColumnGroups(new int[][] { { 1, 5 }, { 3, 7 } });

        PanelBuilder builder = new PanelBuilder(layout);
        builder.setDefaultDialogBorder();

        this.model = new SalesModel();
        JTable itemsTable = new JTable(model);
        JScrollPane scrollPane = new JScrollPane(itemsTable);
        scrollPane.setOpaque(false);
        scrollPane.getViewport().setOpaque(false);
        scrollPane.setViewportBorder(null);

        saleButton = UIHelper.createButton("Perform Sale");
        saleButton.addActionListener(new PerformSaleAction());

        info = new JLabel(" ");
        info.setPreferredSize(new java.awt.Dimension(600, 16));

        CellConstraints cc = new CellConstraints();
        builder.addSeparator("Informations", cc.xyw(1, 1, 7)).setOpaque(false);
        builder.addLabel("Purchase Order:", cc.xy(1, 3));
        builder.add(purchase = new JTextField(15), cc.xy(3, 3));
        builder.addLabel("Order Type:", cc.xy(5, 3));
        builder.add(orderType = new JTextField(15), cc.xy(7, 3));
        builder.addLabel("Organization :", cc.xy(1, 5));
        builder.add(organization = new JTextField(15), cc.xy(3, 5));
        builder.addLabel("Division:", cc.xy(5, 5));
        builder.add(division = new JTextField(15), cc.xy(7, 5));
        builder.addLabel("Channel :", cc.xy(1, 7));
        builder.add(channel = new JTextField(15), cc.xy(3, 7));
        builder.addLabel("Sold To:", cc.xy(1, 9));
        builder.add(soldTo = new JTextField(15), cc.xy(3, 9));

        builder.add(scrollPane, cc.xyw(1, 11, 7));
        builder.add(info, cc.xyw(1, 13, 7, "left, fill"));
        builder.add(saleButton, cc.xyw(1, 15, 7, "center, fill"));

        JPanel form = builder.getPanel();
        form.setOpaque(false);
        setDefaultValues();

        cardPane = new JPanel(carder = new CardLayout());
        cardPane.setOpaque(false);
        cardPane.add("form", form);

        return cardPane;
    }

    /**
     * Donne des valeurs par défauts aux champs du formulaire.
     */
    private void setDefaultValues() {
        soldTo.setText("0000001032");
        channel.setText("10");
        division.setText("00");
        organization.setText("1000");
        orderType.setText("TA");
        purchase.setText("DG-19970626-300");
    }

    public void quit() {
        glassPane.setVisible(false);
        glassPane.interrupt();
    }

    /**
     * Action pour l'exécution de la BAPI de validation de la vente.
     * 
     * @author Romain Guy
     */
    class PerformSaleAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
		            saleButton.setEnabled(false);
                    glassPane.start();
		            Thread performer = new Thread(new Runnable() {
		                public void run() {
		                    perform();
		                }
		            }, "Performer");
		            performer.start();
                }
            });
        }

    }

    /**
     * Modèle de données pour la table des items de vente.
     * 
     * @author Romain Guy
     */
    class SalesModel extends AbstractTableModel {

        private ArrayList items;

        private final String[] columnsNames = new String[] { "Item", "Material", "Quantity" };

        SalesModel() {
            this.items = new ArrayList(15);
        }

        public String[] getItems() {
            return (String[]) items.toArray(new String[0]);
        }

        public String getColumnName(int columnIndex) {
            return columnsNames[columnIndex];
        }

        public int getColumnCount() {
            return columnsNames.length;
        }

        public int getRowCount() {
            return 15;
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            if (rowIndex >= items.size())
                return null;

            String[] item = (String[]) items.get(rowIndex);
            return item[columnIndex];
        }

        public boolean isCellEditable(int rowIndex, int columIndex) {
            return true;
        }

        public void setValueAt(Object value, int rowIndex, int columnIndex) {
            String[] item;
            if (rowIndex >= items.size()) {
                item = new String[getColumnCount()];
                items.add(item);
            } else {
                item = (String[]) items.get(rowIndex);
            }

            item[columnIndex] = value.toString();
            fireTableCellUpdated(rowIndex >= items.size() ? items.size() - 1 : rowIndex, columnIndex);
        }
    }
}
