/*
 * Created on 24 juin 2004
 * UIHelper.java
 * Méthodes utilitaires pour la réalisation d'interfaces graphiques.
 */
package org.progx.salesmanager.ui;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JButton;

import org.progx.salesmanager.SalesManager;


/**
 * Classe utilitaire pour la création d'interfaces graphiques.
 * 
 * @author Romain Guy
 */
public final class UIHelper {

    /**
     * Crée un bouton muni d'un texte.
     * 
     * @param text Le titre du bouton
     * @return Le nouveau bouton
     * @author Romain Guy
     */
    public static JButton createButton(String text) {
        JButton button = new JButton(text);
        button.setFocusPainted(true);
        button.setBorderPainted(true);
        button.setContentAreaFilled(true);
        return button;
    }

    /**
     * Crée un bouton muni d'une image et d'un texte. L'image change lors du survol
     * de la souris.
     * 
     * @param text Le titre du bouton
     * @param icon L'icone associée au bouton. Les icons icon.png, icon_highlight.png et
     *             icon_pressed.png doivent se trouver dans images/.
     * @return Le nouveau bouton
     * @author Romain Guy
     */
    public static JButton createButton(String text, String icon) {
        return createButton(text, icon, false);
    }

    /**
     * Crée un bouton muni d'une image et d'un texte. L'image change lors du survol
     * de la souris.
     * 
     * @param text Le titre du bouton
     * @param icon L'icone associée au bouton. Les icons icon.png, icon_highlight.png et
     *             icon_pressed.png doivent se trouver dans images/.
     * @param flat Si vrai, le bouton sera plat
     * @return Le nouveau bouton
     * @author Romain Guy
     */
    public static JButton createButton(String text, String icon, boolean flat) {
        ImageIcon iconNormal = readImageIcon(icon + ".png");
        ImageIcon iconHighlight = readImageIcon(icon + "_highlight.png");
        ImageIcon iconPressed = readImageIcon(icon + "_pressed.png");

        JButton button = new JButton(text, iconNormal);
        button.setFocusPainted(!flat);
        button.setBorderPainted(!flat);
        button.setContentAreaFilled(!flat);
        if (iconHighlight != null) {
            button.setRolloverEnabled(true);
           button.setRolloverIcon(iconHighlight);
        }
        if (iconPressed != null)
          button.setPressedIcon(iconPressed);
        return button;
    }

    /**
     * Place le composant fourni en paramètre au centre de l'écran.
     * 
     * @param component Le composant à centrer par rapport à l'écran
     * *@author Romain Guy
     */
    public static void centerOnScreen(Component component) {
        Dimension paneSize = component.getSize();
        Dimension screenSize = component.getToolkit().getScreenSize();
        component.setLocation(
            (screenSize.width - paneSize.width) / 2,
            (screenSize.height - paneSize.height) / 2);
    }

    /**
     * Charge le fichier spécifié sous forme d'icone.
     * 
     * @param filename Le nom du fichier image à charger depuis le package
     *        fr.capgemini.salesmanager.images
     * @return Un objet icone
     * @author Romain Guy
     */
    protected static ImageIcon readImageIcon(String filename) {
        URL url = SalesManager.class.getResource("images/" + filename);
        if (url == null)
            return null;

        return new ImageIcon(Toolkit.getDefaultToolkit().getImage(url));
    }
}
