/*
 * Created on 5 juil. 2004
 * ViewSale.java
 * Affichage d'un ordre de vente.
 */
package org.progx.salesmanager.ui;

import java.awt.CardLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.table.AbstractTableModel;

import org.progx.salesmanager.event.QuitListener;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;


/**
 * Affichage d'un ordre de vente.
 * 
 * @author Romain Guy
 */
public class ViewSale implements QuitListener {

    private SalesModel model;
    private AnimatedPanel connectionAnimation;
    private CardLayout carder;
    private JPanel cardPane;
    private JLabel info;
    private JButton searchButton;
    private JTextField orderNumber;
    private JTextField documentDate;
    private JTextField purchaseOrder;
    private JTextField deliveryDate;
    private SalesManagerFrame parent;

    public ViewSale(SalesManagerFrame parent) {
        this.parent = parent;
        this.parent.addQuitListener(this);
    }

    /**
     * Cherche ordre de vente.
     * 
     * @author Romain Guy
     */
    private void search() {
        try {
            Thread.sleep(4000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        displaySuccess("You've successfully waited :)");

        searchButton.setEnabled(true);
        carder.show(cardPane, "form");
        connectionAnimation.stop();
    }

    /**
     * Affiche un message de succès.
     * 
     * @param text Le message à afficher
     */
    private void displaySuccess(String text) {
        info.setIcon(UIHelper.readImageIcon("success.png"));
        info.setText(text);
    }

    /**
     * Affiche un message d'avertissement.
     * 
     * @param text Le message à afficher
     */
    private void displayWarning(String text) {
        info.setIcon(UIHelper.readImageIcon("warning.png"));
        info.setText(text);
    }

    /**
     * Affiche un message d'erreur.
     * 
     * @param text Le message à afficher
     */
    private void displayFailure(String text) {
        info.setIcon(UIHelper.readImageIcon("failure.png"));
        info.setText(text);
    }

    /**
     * Crée un formulaire de visualisation de commande dans un JPanel.
     * 
     * @return Un panneau contenant le formulaire de saisie d'une commande
     * @author Romain Guy
     */
    public Component buildPanel() {
        FormLayout layout =
            new FormLayout("right:pref, 3dlu, pref, 7dlu, right:pref, 3dlu, pref", // colonnes
                "p, 3dlu, p, 9dlu, p, 3dlu, p, 3dlu," + // lignes
                "p, 9dlu, fill:default, 3dlu, p"); // lignes

        PanelBuilder builder = new PanelBuilder(layout);
        builder.setDefaultDialogBorder();

        model = new SalesModel();
        JTable itemsTable = new JTable(model);
        JScrollPane scrollPane = new JScrollPane(itemsTable);
        scrollPane.setOpaque(false);
        scrollPane.getViewport().setOpaque(false);
        scrollPane.setViewportBorder(null);

        searchButton = UIHelper.createButton("Search");
        searchButton.addActionListener(new SearchAction());

        info = new JLabel(" ");
        info.setPreferredSize(new java.awt.Dimension(600, 16));

        CellConstraints cc = new CellConstraints();
        builder.addSeparator("Search", cc.xyw(1, 1, 7)).setOpaque(false);
        builder.addLabel("Order Number:", cc.xy(1, 3));
        builder.add(orderNumber = new JTextField(15), cc.xy(3, 3));
        builder.add(searchButton, cc.xy(5, 3, "left, default"));
        builder.addSeparator("Result", cc.xyw(1, 5, 7)).setOpaque(false);
        builder.addLabel("Date:", cc.xy(1, 7));
        builder.add(documentDate = new JTextField(15), cc.xy(3, 7));
        builder.addLabel("Delivery Date:", cc.xy(5, 7));
        builder.add(deliveryDate = new JTextField(15), cc.xy(7, 7));
        builder.addLabel("Purchase Order:", cc.xy(1, 9));
        builder.add(purchaseOrder = new JTextField(15), cc.xy(3, 9));
        builder.add(scrollPane, cc.xyw(1, 11, 7));
        builder.add(info, cc.xyw(1, 13, 7, "left, fill"));

        documentDate.setEnabled(false);
        deliveryDate.setEnabled(false);
        purchaseOrder.setEnabled(false);
        //itemsTable.setEnabled(false);

        JPanel form = builder.getPanel();
        form.setOpaque(false);
        setDefaultValues();

        cardPane = new JPanel(carder = new CardLayout());
        cardPane.setOpaque(false);
        cardPane.add("form", form);

        connectionAnimation =
            new AnimatedPanel(
                "Sending the request to the remote system...",
                UIHelper.readImageIcon("network.png"));
        cardPane.add("animation", connectionAnimation);

        return cardPane;
    }

    /**
     * Donne des valeurs par défauts aux champs du formulaire.
     */
    private void setDefaultValues() {
        orderNumber.setText("0000006973");
    }

    /**
     * Réinitialise le formulaire.
     */
    private void resetValues() {
        documentDate.setText("");
        deliveryDate.setText("");
        purchaseOrder.setText("");
        model.reset();
    }

    public void quit() {
        connectionAnimation.stop();
    }

    /**
     * Action pour l'exécution d'une recherche d'une vente.
     * 
     * @author Romain Guy
     */
    public class SearchAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
		            carder.show(cardPane, "animation");
		            connectionAnimation.start();
		            resetValues();
		            searchButton.setEnabled(false);
		            Thread performer = new Thread(new Runnable() {
		                public void run() {
		                    search();
		                }
		            }, "Searcher");
		            performer.start();
                }
            });
        }

    }

    /**
     * Modèle de données pour la table des items de vente.
     * 
     * @author Romain Guy
     */
    class SalesModel extends AbstractTableModel {

        private ArrayList items;

        private final String[] columnsNames = new String[] { "Item", "Material", "Quantity" };

        SalesModel() {
            this.items = new ArrayList(15);
        }
        
        public void reset() {
            this.items.clear();
        }

        public String[] getItems() {
            return (String[]) items.toArray(new String[0]);
        }

        public String getColumnName(int columnIndex) {
            return columnsNames[columnIndex];
        }

        public int getColumnCount() {
            return columnsNames.length;
        }

        public int getRowCount() {
            return 15;
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            if (rowIndex >= items.size())
                return null;

            String[] item = (String[]) items.get(rowIndex);
            return item[columnIndex];
        }

        public boolean isCellEditable(int rowIndex, int columIndex) {
            return true;
        }

        public void setValueAt(Object value, int rowIndex, int columnIndex) {
            String[] item;
            if (rowIndex >= items.size()) {
                item = new String[getColumnCount()];
                items.add(item);
            } else {
                item = (String[]) items.get(rowIndex);
            }

            item[columnIndex] = value.toString();
            fireTableCellUpdated(rowIndex >= items.size() ? items.size() - 1 : rowIndex, columnIndex);
        }
    }
}
